// The following ifdef block is the standard way of creating macros which make exporting 
// from a DLL simpler. All files within this DLL are compiled with the ATLAS_EXPORTS
// symbol defined on the command line. this symbol should not be defined on any project
// that uses this DLL. This way any other project whose source files include this file see 
// ATLAS_API functions as being imported from a DLL, whereas this DLL sees symbols
// defined with this macro as being exported.
#ifdef ATLAS_EXPORTS
#define ATLAS_API __declspec(dllexport)
#else
#define ATLAS_API __declspec(dllimport)
#endif

/*! Callback function type for distance calculation. */
typedef double (* AT_CalculateDistanceCallbackType) (int vID0, int vID1);

/*!
  \defgroup Vertex Vertex functions
*/
/*!
  \defgroup Edge Edge functions
*/
/*!
  \defgroup Query Query functions
*/
/*!
  \defgroup Misc Misc functions
*/

//! Create and maintain a graph and find shortest paths.
/*! */

/*! The function was succesful. */
#define AT_SUCCESS                  0
/*! You provided an illegal value for one or more of the parameters. */
#define AT_ILLEGAL_VALUE           -1
/*! You try to add an edge that already exists. */
#define AT_EDGE_ALREADY_EXISTS     -2
/*! The query did not yield a path. */
#define AT_NO_PATH_FOUND           -3
/*! The vertex ID does not exist. */
#define AT_NO_SUCH_VERTEX          -4
/*! The edge does not exist. */
#define AT_NO_SUCH_EDGE            -5
/*! The operation you try to perform is only valid on undirected graphs. */
#define AT_GRAPH_DIRECTED          -6
/*! You changed the graph and need to recalculate the shortest paths before querying. */
#define AT_NEED_RECALCULATION      -7

//! These are all functions exported by GURP.
class ATLAS_API CAtlas {
public:
  /*! This function creates an instance of Atlas.
	  \param dimension The dimension of the vertices.
    \param directed Set to TRUE if the graph is directed.
	  \returns The function returns AT_SUCCESS on success, and an errorcode on failure.
    \ingroup Misc
  */
  CAtlas                     (int dimension, bool directed);
  /*! The destructor for the class.
    \ingroup Misc
  */
  virtual ~CAtlas            ();

  // vertex functions
  /*! This function adds a vertex to the graph.
	  \param conf A list of length dimension containing the values of the vertex.
    \param vID A return value contaning the ID of the vertex.
    \param userdef An optional pointer to store user defined data with the vertex.
	  \returns The function returns AT_SUCCESS on success, and an errorcode on failure.
    \ingroup Vertex
  */
  int AddVertex              (double *conf, int *vID, void *userdef=NULL);
  /*! This function removes a vertex from the graph.
	  \param vID The ID of the vertex to remove.
	  \returns The function returns AT_SUCCESS on success, and an errorcode on failure.
    \ingroup Vertex
  */
  int RemoveVertex           (int vID);
  /*! Use this function to retrieve information from the vertex.
	  \param vID The ID of the vertex.
    \param conf Return value: the list of vertex values, must be of the proper size, can be NULL if you are only interested in userdef.
    \param userdef Return value: optional, the user defined pointer associated with the vertex.
	  \returns The function returns AT_SUCCESS on success, and an errorcode on failure.
    \ingroup Vertex
  */
  int GetVertexConfiguration (int vID, double* conf, void **userdef=NULL);
  /*! This function retrieves the number of neighbors of a vertex.
	  \param vID The ID of the vertex.
    \param nr The number of neighbors of the vertex.
	  \returns The function returns AT_SUCCESS on success, and an errorcode on failure.
    \ingroup Vertex
  */
  int GetNrNeighbors         (int vID, int *nr);
  /*! This function retrieves neighbors of a vertex.
	  \param vID The ID of the vertex.
    \param vIDs The list of neighbor vertex ID's. The list needs to have the proper length.
	  \returns The function returns AT_SUCCESS on success, and an errorcode on failure.
    \ingroup Vertex
  */
  int GetNeighbors           (int vID, int *vIDs);
  /*! This function retrieves the total number of vertices in the graph.
	  \param nr The number of vertices.
	  \returns The function returns AT_SUCCESS on success, and an errorcode on failure.
    \ingroup Vertex
  */
  int GetNrVertices          (int *nr);
  /*! This function retrieves the vertices in the graph.
	  \param vIDs The list of vertices. This list needs to have the proper size.
	  \returns The function returns AT_SUCCESS on success, and an errorcode on failure.
    \ingroup Vertex
  */
  int GetVertices            (int *vIDs);
  /*! This function enables the user to set the user defined pointer at a later moment.
	  \param vID The ID of the vertex.
    \param userdef The pointer to the user defined data.
	  \returns The function returns AT_SUCCESS on success, and an errorcode on failure.
    \ingroup Vertex
  */
  int SetUserDef             (int vID, void *userdef);

  // edge functions
  /*! Adds a connection between two vertices.
	  \param vID0 The ID of the first vertex.
    \param vID1 The ID of the second vertex.
    \param value The value of the edge, used by finding shortest paths.
	  \returns The function returns AT_SUCCESS on success, and an errorcode on failure.
    \ingroup Edge
  */
  int AddEdge                (int vID0, int vID1, double value);
  /*! This function removes a connection from the graph.
	  \param vID0 The ID of the first vertex.
    \param vID1 The ID of the second vertex.
	  \returns The function returns AT_SUCCESS on success, and an errorcode on failure.
    \ingroup Vertex
  */
  int RemoveEdge             (int vID0, int vID1);
  /*! This function sets the value of an edge.
	  \param vID0 The ID of the first vertex.
    \param vID1 The ID of the second vertex.
    \param value The new value for the edge.
	  \returns The function returns AT_SUCCESS on success, and an errorcode on failure.
    \ingroup Vertex
  */
  int SetEdgeValue           (int vID0, int vID1, double value);
  /*! This function retrieves the value of an edge.
	  \param vID0 The ID of the first vertex.
    \param vID1 The ID of the second vertex.
    \param value The value of the edge.
	  \returns The function returns AT_SUCCESS on success, and an errorcode on failure.
    \ingroup Vertex
  */
  int GetEdgeValue           (int vID0, int vID1, double *value);
  // query functions
  /*! This function calculates a single source shortest path.
	  \param startID The ID of the vertex from which to calculate the shortest paths.
	  \returns The function returns AT_SUCCESS on success, and an errorcode on failure.
    \ingroup Query
  */
  int FindShortestPaths      (int startID);
  /*! Query for a shortest path. First use FindShortestPaths!
	  \param goalID The ID of the goal vertex.
    \param size Return value for the number of vertices in the query path.
    \param length Return value that holds the length of the path.
	  \returns The function returns AT_SUCCESS on success, and an errorcode on failure.
    \ingroup Vertex
  */
  int Query                  (int goalID, int *size, double *length=NULL);
  /*! Retrieve a vertex in the query.
	  \param nr The number of the query vertex. The total numer is returned by the Query function.
    \param vID Retur value for the ID of the vertex in the query.
	  \returns The function returns AT_SUCCESS on success, and an errorcode on failure.
    \ingroup Vertex
  */
  int GetQueryVertex         (int nr, int* vID);
  /*! Check whether two vertices are connected, either direct or via other vertices.
	  \param vID0 The ID of the first vertex.
    \param vID1 The ID of the second vertex.
    \param connected Return value whether the two vertices are connected.
	  \returns The function returns AT_SUCCESS on success, and an errorcode on failure.
    \ingroup Vertex
  */
  int VerticesConnected      (int vID0, int vID1, bool *connected);

  //misc
  /*! Delete graph.
	  \returns The function returns AT_SUCCESS on success, and an errorcode on failure.
    \ingroup Misc
  */
  int ResetGraph             ();
  /*! Loads a graph from disc.
    \param fileName Name of the file to load.
	  \returns The function returns AT_SUCCESS on success, and an errorcode on failure.
    \ingroup Misc
  */
  int LoadGraph              (char *fileName);
  /*! Saves a graph to disc.
    \param fileName Name of the file to save.
	  \returns The function returns AT_SUCCESS on success, and an errorcode on failure.
    \ingroup Misc
  */
  int SaveGraph              (char *fileName);

  /*! Checks whether an edge that creates a cycle is "useful".
    \param startID The ID of the start vertex.
    \param goalID The ID of the goal vertex.
    \param K The K value to uses when determining the usefulness.
    \param calcDistCB A user provided callback routine to calculate the discance between two vertices.
    \param result True when the edge is useful.
    \returns The function returns AT_SUCCESS on success, and an errorcode on failure.
    \ingroup Misc
  */
  int CheckUsefulCycle       (int startID, int goalID, double K, AT_CalculateDistanceCallbackType calcDistCB, bool *result);
};

/*! \file atlas.h
*/